# This file is autogenerated by the command `make fix-copies`, do not edit.
from ..file_utils import requires_backends


class TensorFlowBenchmarkArguments:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TensorFlowBenchmark:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


def tf_top_k_top_p_filtering(*args, **kwargs):
    requires_backends(tf_top_k_top_p_filtering, ["tf"])


class PushToHubCallback:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_LAYOUTLM_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFLayoutLMForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLayoutLMForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLayoutLMForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLayoutLMMainLayer:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLayoutLMModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLayoutLMPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFSequenceSummary:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFSharedEmbeddings:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


def shape_list(*args, **kwargs):
    requires_backends(shape_list, ["tf"])


TF_ALBERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFAlbertForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAlbertForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAlbertForPreTraining:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAlbertForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAlbertForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAlbertForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAlbertMainLayer:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAlbertModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAlbertPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_MODEL_FOR_CAUSAL_LM_MAPPING = None


TF_MODEL_FOR_IMAGE_CLASSIFICATION_MAPPING = None


TF_MODEL_FOR_MASKED_LM_MAPPING = None


TF_MODEL_FOR_MULTIPLE_CHOICE_MAPPING = None


TF_MODEL_FOR_NEXT_SENTENCE_PREDICTION_MAPPING = None


TF_MODEL_FOR_PRETRAINING_MAPPING = None


TF_MODEL_FOR_QUESTION_ANSWERING_MAPPING = None


TF_MODEL_FOR_SEQ_TO_SEQ_CAUSAL_LM_MAPPING = None


TF_MODEL_FOR_SEQUENCE_CLASSIFICATION_MAPPING = None


TF_MODEL_FOR_TABLE_QUESTION_ANSWERING_MAPPING = None


TF_MODEL_FOR_TOKEN_CLASSIFICATION_MAPPING = None


TF_MODEL_MAPPING = None


TF_MODEL_WITH_LM_HEAD_MAPPING = None


class TFAutoModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForCausalLM:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForImageClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForPreTraining:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForSeq2SeqLM:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForTableQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFAutoModelWithLMHead:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBartForConditionalGeneration:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBartModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBartPretrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_BERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFBertEmbeddings:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBertForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBertForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBertForNextSentencePrediction:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBertForPreTraining:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBertForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBertForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBertForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBertLMHeadModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBertMainLayer:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBertModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBertPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBlenderbotForConditionalGeneration:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBlenderbotModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBlenderbotPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBlenderbotSmallForConditionalGeneration:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBlenderbotSmallModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFBlenderbotSmallPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_CAMEMBERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFCamembertForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFCamembertForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFCamembertForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFCamembertForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFCamembertForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFCamembertModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_CONVBERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFConvBertForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFConvBertForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFConvBertForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFConvBertForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFConvBertForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFConvBertLayer:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFConvBertModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFConvBertPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_CTRL_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFCTRLForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFCTRLLMHeadModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFCTRLModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFCTRLPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_DEBERTA_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFDebertaForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDebertaForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDebertaForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDebertaForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDebertaModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDebertaPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_DEBERTA_V2_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFDebertaV2ForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDebertaV2ForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDebertaV2ForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDebertaV2ForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDebertaV2Model:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDebertaV2PreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_DISTILBERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFDistilBertForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDistilBertForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDistilBertForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDistilBertForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDistilBertForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDistilBertMainLayer:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDistilBertModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDistilBertPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_DPR_CONTEXT_ENCODER_PRETRAINED_MODEL_ARCHIVE_LIST = None


TF_DPR_QUESTION_ENCODER_PRETRAINED_MODEL_ARCHIVE_LIST = None


TF_DPR_READER_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFDPRContextEncoder:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDPRPretrainedContextEncoder:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDPRPretrainedQuestionEncoder:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDPRPretrainedReader:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDPRQuestionEncoder:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFDPRReader:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_ELECTRA_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFElectraForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFElectraForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFElectraForPreTraining:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFElectraForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFElectraForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFElectraForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFElectraModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFElectraPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFEncoderDecoderModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_FLAUBERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFFlaubertForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFlaubertForQuestionAnsweringSimple:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFlaubertForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFlaubertForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFlaubertModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFlaubertPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFlaubertWithLMHeadModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_FUNNEL_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFFunnelBaseModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFunnelForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFunnelForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFunnelForPreTraining:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFunnelForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFunnelForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFunnelForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFunnelModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFFunnelPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_GPT2_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFGPT2DoubleHeadsModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFGPT2ForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFGPT2LMHeadModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFGPT2MainLayer:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFGPT2Model:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFGPT2PreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_HUBERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFHubertForCTC:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFHubertModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFHubertPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLEDForConditionalGeneration:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLEDModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLEDPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_LONGFORMER_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFLongformerForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLongformerForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLongformerForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLongformerForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLongformerForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLongformerModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLongformerPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLongformerSelfAttention:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_LXMERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFLxmertForPreTraining:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLxmertMainLayer:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLxmertModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLxmertPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFLxmertVisualFeatureEncoder:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMarianModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMarianMTModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMarianPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMBartForConditionalGeneration:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMBartModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMBartPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_MOBILEBERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFMobileBertForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMobileBertForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMobileBertForNextSentencePrediction:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMobileBertForPreTraining:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMobileBertForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMobileBertForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMobileBertForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMobileBertMainLayer:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMobileBertModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMobileBertPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_MPNET_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFMPNetForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMPNetForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMPNetForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMPNetForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMPNetForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMPNetMainLayer:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMPNetModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMPNetPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMT5EncoderModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMT5ForConditionalGeneration:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFMT5Model:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_OPENAI_GPT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFOpenAIGPTDoubleHeadsModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFOpenAIGPTForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFOpenAIGPTLMHeadModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFOpenAIGPTMainLayer:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFOpenAIGPTModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFOpenAIGPTPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFPegasusForConditionalGeneration:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFPegasusModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFPegasusPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRagModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRagPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRagSequenceForGeneration:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRagTokenForGeneration:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_REMBERT_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFRemBertForCausalLM:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRemBertForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRemBertForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRemBertForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRemBertForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRemBertForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRemBertLayer:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRemBertModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRemBertPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_ROBERTA_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFRobertaForCausalLM:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRobertaForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRobertaForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRobertaForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRobertaForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRobertaForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRobertaMainLayer:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRobertaModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRobertaPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_ROFORMER_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFRoFormerForCausalLM:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRoFormerForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRoFormerForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRoFormerForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRoFormerForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRoFormerForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRoFormerLayer:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRoFormerModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFRoFormerPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_T5_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFT5EncoderModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFT5ForConditionalGeneration:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFT5Model:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFT5PreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_TAPAS_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFTapasForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFTapasForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFTapasForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFTapasModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFTapasPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_TRANSFO_XL_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFAdaptiveEmbedding:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFTransfoXLForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFTransfoXLLMHeadModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFTransfoXLMainLayer:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFTransfoXLModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFTransfoXLPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFViTForImageClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFViTModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFViTPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_WAV_2_VEC_2_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFWav2Vec2ForCTC:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFWav2Vec2Model:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFWav2Vec2PreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_XLM_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFXLMForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLMForQuestionAnsweringSimple:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLMForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLMForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLMMainLayer:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLMModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLMPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLMWithLMHeadModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_XLM_ROBERTA_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFXLMRobertaForMaskedLM:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLMRobertaForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLMRobertaForQuestionAnswering:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLMRobertaForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLMRobertaForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLMRobertaModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


TF_XLNET_PRETRAINED_MODEL_ARCHIVE_LIST = None


class TFXLNetForMultipleChoice:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLNetForQuestionAnsweringSimple:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLNetForSequenceClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLNetForTokenClassification:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLNetLMHeadModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLNetMainLayer:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLNetModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class TFXLNetPreTrainedModel:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["tf"])

    def call(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class AdamWeightDecay:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class GradientAccumulator:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


class WarmUp:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])


def create_optimizer(*args, **kwargs):
    requires_backends(create_optimizer, ["tf"])


class TFTrainer:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["tf"])



# coding=utf-8
# Copyright 2020 The Allen Institute for AI team and The HuggingFace Inc. team.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ...utils import logging
from ..roberta.tokenization_roberta_fast import RobertaTokenizerFast
from .tokenization_longformer import LongformerTokenizer


logger = logging.get_logger(__name__)


VOCAB_FILES_NAMES = {"vocab_file": "vocab.json", "merges_file": "merges.txt", "tokenizer_file": "tokenizer.json"}

PRETRAINED_VOCAB_FILES_MAP = {
    "vocab_file": {
        "allenai/longformer-base-4096": "https://huggingface.co/allenai/longformer-base-4096/resolve/main/vocab.json",
        "allenai/longformer-large-4096": "https://huggingface.co/allenai/longformer-large-4096/resolve/main/vocab.json",
        "allenai/longformer-large-4096-finetuned-triviaqa": "https://huggingface.co/allenai/longformer-large-4096-finetuned-triviaqa/resolve/main/vocab.json",
        "allenai/longformer-base-4096-extra.pos.embd.only": "https://huggingface.co/allenai/longformer-base-4096-extra.pos.embd.only/resolve/main/vocab.json",
        "allenai/longformer-large-4096-extra.pos.embd.only": "https://huggingface.co/allenai/longformer-large-4096-extra.pos.embd.only/resolve/main/vocab.json",
    },
    "merges_file": {
        "allenai/longformer-base-4096": "https://huggingface.co/allenai/longformer-base-4096/resolve/main/merges.txt",
        "allenai/longformer-large-4096": "https://huggingface.co/allenai/longformer-large-4096/resolve/main/merges.txt",
        "allenai/longformer-large-4096-finetuned-triviaqa": "https://huggingface.co/allenai/longformer-large-4096-finetuned-triviaqa/resolve/main/merges.txt",
        "allenai/longformer-base-4096-extra.pos.embd.only": "https://huggingface.co/allenai/longformer-base-4096-extra.pos.embd.only/resolve/main/merges.txt",
        "allenai/longformer-large-4096-extra.pos.embd.only": "https://huggingface.co/allenai/longformer-large-4096-extra.pos.embd.only/resolve/main/merges.txt",
    },
    "tokenizer_file": {
        "allenai/longformer-base-4096": "https://huggingface.co/allenai/longformer-base-4096/resolve/main/tokenizer.json",
        "allenai/longformer-large-4096": "https://huggingface.co/allenai/longformer-large-4096/resolve/main/tokenizer.json",
        "allenai/longformer-large-4096-finetuned-triviaqa": "https://huggingface.co/allenai/longformer-large-4096-finetuned-triviaqa/resolve/main/tokenizer.json",
        "allenai/longformer-base-4096-extra.pos.embd.only": "https://huggingface.co/allenai/longformer-base-4096-extra.pos.embd.only/resolve/main/tokenizer.json",
        "allenai/longformer-large-4096-extra.pos.embd.only": "https://huggingface.co/allenai/longformer-large-4096-extra.pos.embd.only/resolve/main/tokenizer.json",
    },
}

PRETRAINED_POSITIONAL_EMBEDDINGS_SIZES = {
    "allenai/longformer-base-4096": 4096,
    "allenai/longformer-large-4096": 4096,
    "allenai/longformer-large-4096-finetuned-triviaqa": 4096,
    "allenai/longformer-base-4096-extra.pos.embd.only": 4096,
    "allenai/longformer-large-4096-extra.pos.embd.only": 4096,
}


class LongformerTokenizerFast(RobertaTokenizerFast):
    r"""
    Construct a "fast" Longformer tokenizer (backed by HuggingFace's `tokenizers` library).

    :class:`~transformers.LongformerTokenizerFast` is identical to :class:`~transformers.RobertaTokenizerFast`. Refer
    to the superclass for usage examples and documentation concerning parameters.
    """
    # merges and vocab same as Roberta
    vocab_files_names = VOCAB_FILES_NAMES
    pretrained_vocab_files_map = PRETRAINED_VOCAB_FILES_MAP
    max_model_input_sizes = PRETRAINED_POSITIONAL_EMBEDDINGS_SIZES
    slow_tokenizer_class = LongformerTokenizer



# Copyright 2020 The HuggingFace Team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from argparse import ArgumentParser
from os import listdir, makedirs
from pathlib import Path
from typing import Dict, List, Optional, Tuple

from packaging.version import Version, parse

from transformers.file_utils import ModelOutput, is_tf_available, is_torch_available
from transformers.pipelines import Pipeline, pipeline
from transformers.tokenization_utils import BatchEncoding


# This is the minimal required version to
# support some ONNX Runtime features
ORT_QUANTIZE_MINIMUM_VERSION = parse("1.4.0")


SUPPORTED_PIPELINES = [
    "feature-extraction",
    "ner",
    "sentiment-analysis",
    "fill-mask",
    "question-answering",
    "text-generation",
    "translation_en_to_fr",
    "translation_en_to_de",
    "translation_en_to_ro",
]


class OnnxConverterArgumentParser(ArgumentParser):
    """
    Wraps all the script arguments supported to export transformers models to ONNX IR
    """

    def __init__(self):
        super().__init__("ONNX Converter")

        self.add_argument(
            "--pipeline",
            type=str,
            choices=SUPPORTED_PIPELINES,
            default="feature-extraction",
        )
        self.add_argument(
            "--model",
            type=str,
            required=True,
            help="Model's id or path (ex: bert-base-cased)",
        )
        self.add_argument("--tokenizer", type=str, help="Tokenizer's id or path (ex: bert-base-cased)")
        self.add_argument(
            "--framework",
            type=str,
            choices=["pt", "tf"],
            help="Framework for loading the model",
        )
        self.add_argument("--opset", type=int, default=11, help="ONNX opset to use")
        self.add_argument(
            "--check-loading",
            action="store_true",
            help="Check ONNX is able to load the model",
        )
        self.add_argument(
            "--use-external-format",
            action="store_true",
            help="Allow exporting model >= than 2Gb",
        )
        self.add_argument(
            "--quantize",
            action="store_true",
            help="Quantize the neural network to be run with int8",
        )
        self.add_argument("output")


def generate_identified_filename(filename: Path, identifier: str) -> Path:
    """
    Append a string-identifier at the end (before the extension, if any) to the provided filepath

    Args:
        filename: pathlib.Path The actual path object we would like to add an identifier suffix
        identifier: The suffix to add

    Returns: String with concatenated identifier at the end of the filename
    """
    return filename.parent.joinpath(filename.stem + identifier).with_suffix(filename.suffix)


def check_onnxruntime_requirements(minimum_version: Version):
    """
    Check onnxruntime is installed and if the installed version match is recent enough

    Raises:
        ImportError: If onnxruntime is not installed or too old version is found
    """
    try:
        import onnxruntime

        # Parse the version of the installed onnxruntime
        ort_version = parse(onnxruntime.__version__)

        # We require 1.4.0 minimum
        if ort_version < ORT_QUANTIZE_MINIMUM_VERSION:
            raise ImportError(
                f"We found an older version of onnxruntime ({onnxruntime.__version__}) "
                f"but we require onnxruntime to be >= {minimum_version} to enable all the conversions options.\n"
                f"Please update onnxruntime by running `pip install --upgrade onnxruntime`"
            )

    except ImportError:
        raise ImportError(
            "onnxruntime doesn't seem to be currently installed. "
            "Please install the onnxruntime by running `pip install onnxruntime`"
            " and relaunch the conversion."
        )


def ensure_valid_input(model, tokens, input_names):
    """
    Ensure input are presented in the correct order, without any Non

    Args:
        model: The model used to forward the input data
        tokens: BatchEncoding holding the input data
        input_names: The name of the inputs

    Returns: Tuple

    """
    print("Ensuring inputs are in correct order")

    model_args_name = model.forward.__code__.co_varnames
    model_args, ordered_input_names = [], []
    for arg_name in model_args_name[1:]:  # start at index 1 to skip "self" argument
        if arg_name in input_names:
            ordered_input_names.append(arg_name)
            model_args.append(tokens[arg_name])
        else:
            print(f"{arg_name} is not present in the generated input list.")
            break

    print(f"Generated inputs order: {ordered_input_names}")
    return ordered_input_names, tuple(model_args)


def infer_shapes(nlp: Pipeline, framework: str) -> Tuple[List[str], List[str], Dict, BatchEncoding]:
    """
    Attempt to infer the static vs dynamic axes for each input and output tensors for a specific model

    Args:
        nlp: The pipeline object holding the model to be exported
        framework: The framework identifier to dispatch to the correct inference scheme (pt/tf)

    Returns:

        - List of the inferred input variable names
        - List of the inferred output variable names
        - Dictionary with input/output variables names as key and shape tensor as value
        - a BatchEncoding reference which was used to infer all the above information
    """

    def build_shape_dict(name: str, tensor, is_input: bool, seq_len: int):
        if isinstance(tensor, (tuple, list)):
            return [build_shape_dict(name, t, is_input, seq_len) for t in tensor]

        else:
            # Let's assume batch is the first axis with only 1 element (~~ might not be always true ...)
            axes = {[axis for axis, numel in enumerate(tensor.shape) if numel == 1][0]: "batch"}
            if is_input:
                if len(tensor.shape) == 2:
                    axes[1] = "sequence"
                else:
                    raise ValueError(f"Unable to infer tensor axes ({len(tensor.shape)})")
            else:
                seq_axes = [dim for dim, shape in enumerate(tensor.shape) if shape == seq_len]
                axes.update({dim: "sequence" for dim in seq_axes})

        print(f"Found {'input' if is_input else 'output'} {name} with shape: {axes}")
        return axes

    tokens = nlp.tokenizer("This is a sample output", return_tensors=framework)
    seq_len = tokens.input_ids.shape[-1]
    outputs = nlp.model(**tokens) if framework == "pt" else nlp.model(tokens)
    if isinstance(outputs, ModelOutput):
        outputs = outputs.to_tuple()
    if not isinstance(outputs, (list, tuple)):
        outputs = (outputs,)

    # Generate input names & axes
    input_vars = list(tokens.keys())
    input_dynamic_axes = {k: build_shape_dict(k, v, True, seq_len) for k, v in tokens.items()}

    # flatten potentially grouped outputs (past for gpt2, attentions)
    outputs_flat = []
    for output in outputs:
        if isinstance(output, (tuple, list)):
            outputs_flat.extend(output)
        else:
            outputs_flat.append(output)

    # Generate output names & axes
    output_names = [f"output_{i}" for i in range(len(outputs_flat))]
    output_dynamic_axes = {k: build_shape_dict(k, v, False, seq_len) for k, v in zip(output_names, outputs_flat)}

    # Create the aggregated axes representation
    dynamic_axes = dict(input_dynamic_axes, **output_dynamic_axes)
    return input_vars, output_names, dynamic_axes, tokens


def load_graph_from_args(
    pipeline_name: str, framework: str, model: str, tokenizer: Optional[str] = None, **models_kwargs
) -> Pipeline:
    """
    Convert the set of arguments provided through the CLI to an actual pipeline reference (tokenizer + model

    Args:
        pipeline_name: The kind of pipeline to use (ner, question-answering, etc.)
        framework: The actual model to convert the pipeline from ("pt" or "tf")
        model: The model name which will be loaded by the pipeline
        tokenizer: The tokenizer name which will be loaded by the pipeline, default to the model's value

    Returns: Pipeline object

    """
    # If no tokenizer provided
    if tokenizer is None:
        tokenizer = model

    # Check the wanted framework is available
    if framework == "pt" and not is_torch_available():
        raise Exception("Cannot convert because PyTorch is not installed. Please install torch first.")
    if framework == "tf" and not is_tf_available():
        raise Exception("Cannot convert because TF is not installed. Please install tensorflow first.")

    print(f"Loading pipeline (model: {model}, tokenizer: {tokenizer})")

    # Allocate tokenizer and model
    return pipeline(pipeline_name, model=model, tokenizer=tokenizer, framework=framework, model_kwargs=models_kwargs)


def convert_pytorch(nlp: Pipeline, opset: int, output: Path, use_external_format: bool):
    """
    Export a PyTorch backed pipeline to ONNX Intermediate Representation (IR

    Args:
        nlp: The pipeline to be exported
        opset: The actual version of the ONNX operator set to use
        output: Path where will be stored the generated ONNX model
        use_external_format: Split the model definition from its parameters to allow model bigger than 2GB

    Returns:

    """
    if not is_torch_available():
        raise Exception("Cannot convert because PyTorch is not installed. Please install torch first.")

    import torch
    from torch.onnx import export

    print(f"Using framework PyTorch: {torch.__version__}")

    with torch.no_grad():
        input_names, output_names, dynamic_axes, tokens = infer_shapes(nlp, "pt")
        ordered_input_names, model_args = ensure_valid_input(nlp.model, tokens, input_names)

        export(
            nlp.model,
            model_args,
            f=output.as_posix(),
            input_names=ordered_input_names,
            output_names=output_names,
            dynamic_axes=dynamic_axes,
            do_constant_folding=True,
            use_external_data_format=use_external_format,
            enable_onnx_checker=True,
            opset_version=opset,
        )


def convert_tensorflow(nlp: Pipeline, opset: int, output: Path):
    """
    Export a TensorFlow backed pipeline to ONNX Intermediate Representation (IR

    Args:
        nlp: The pipeline to be exported
        opset: The actual version of the ONNX operator set to use
        output: Path where will be stored the generated ONNX model

    Notes: TensorFlow cannot export model bigger than 2GB due to internal constraint from TensorFlow

    """
    if not is_tf_available():
        raise Exception("Cannot convert because TF is not installed. Please install tensorflow first.")

    print("/!\\ Please note TensorFlow doesn't support exporting model > 2Gb /!\\")

    try:
        import tensorflow as tf

        from keras2onnx import __version__ as k2ov
        from keras2onnx import convert_keras, save_model

        print(f"Using framework TensorFlow: {tf.version.VERSION}, keras2onnx: {k2ov}")

        # Build
        input_names, output_names, dynamic_axes, tokens = infer_shapes(nlp, "tf")

        # Forward
        nlp.model.predict(tokens.data)
        onnx_model = convert_keras(nlp.model, nlp.model.name, target_opset=opset)
        save_model(onnx_model, output.as_posix())

    except ImportError as e:
        raise Exception(f"Cannot import {e.name} required to convert TF model to ONNX. Please install {e.name} first.")


def convert(
    framework: str,
    model: str,
    output: Path,
    opset: int,
    tokenizer: Optional[str] = None,
    use_external_format: bool = False,
    pipeline_name: str = "feature-extraction",
    **model_kwargs
):
    """
    Convert the pipeline object to the ONNX Intermediate Representation (IR) format

    Args:
        framework: The framework the pipeline is backed by ("pt" or "tf")
        model: The name of the model to load for the pipeline
        output: The path where the ONNX graph will be stored
        opset: The actual version of the ONNX operator set to use
        tokenizer: The name of the model to load for the pipeline, default to the model's name if not provided
        use_external_format: Split the model definition from its parameters to allow model bigger than 2GB (PyTorch only)
        pipeline_name: The kind of pipeline to instantiate (ner, question-answering, etc.)
        model_kwargs: Keyword arguments to be forwarded to the model constructor

    Returns:

    """
    print(f"ONNX opset version set to: {opset}")

    # Load the pipeline
    nlp = load_graph_from_args(pipeline_name, framework, model, tokenizer, **model_kwargs)

    if not output.parent.exists():
        print(f"Creating folder {output.parent}")
        makedirs(output.parent.as_posix())
    elif len(listdir(output.parent.as_posix())) > 0:
        raise Exception(f"Folder {output.parent.as_posix()} is not empty, aborting conversion")

    # Export the graph
    if framework == "pt":
        convert_pytorch(nlp, opset, output, use_external_format)
    else:
        convert_tensorflow(nlp, opset, output)


def optimize(onnx_model_path: Path) -> Path:
    """
    Load the model at the specified path and let onnxruntime look at transformations on the graph to enable all the
    optimizations possibl

    Args:
        onnx_model_path: filepath where the model binary description is stored

    Returns: Path where the optimized model binary description has been saved

    """
    from onnxruntime import InferenceSession, SessionOptions

    # Generate model name with suffix "optimized"
    opt_model_path = generate_identified_filename(onnx_model_path, "-optimized")
    sess_option = SessionOptions()
    sess_option.optimized_model_filepath = opt_model_path.as_posix()
    _ = InferenceSession(onnx_model_path.as_posix(), sess_option)

    print(f"Optimized model has been written at {opt_model_path}: \N{heavy check mark}")
    print("/!\\ Optimized model contains hardware specific operators which might not be portable. /!\\")

    return opt_model_path


def quantize(onnx_model_path: Path) -> Path:
    """
    Quantize the weights of the model from float32 to in8 to allow very efficient inference on modern CPU

    Args:
        onnx_model_path: Path to location the exported ONNX model is stored

    Returns: The Path generated for the quantized
    """
    import onnx
    from onnxruntime.quantization import QuantizationMode, quantize

    onnx_model = onnx.load(onnx_model_path.as_posix())

    # Discussed with @yufenglee from ONNX runtime, this will be address in the next release of onnxruntime
    print(
        "As of onnxruntime 1.4.0, models larger than 2GB will fail to quantize due to protobuf constraint.\n"
        "This limitation will be removed in the next release of onnxruntime."
    )

    quantized_model = quantize(
        model=onnx_model,
        quantization_mode=QuantizationMode.IntegerOps,
        force_fusions=True,
        symmetric_weight=True,
    )

    # Append "-quantized" at the end of the model's name
    quantized_model_path = generate_identified_filename(onnx_model_path, "-quantized")

    # Save model
    print(f"Quantized model has been written at {quantized_model_path}: \N{heavy check mark}")
    onnx.save_model(quantized_model, quantized_model_path.as_posix())

    return quantized_model_path


def verify(path: Path):
    from onnxruntime import InferenceSession, SessionOptions
    from onnxruntime.capi.onnxruntime_pybind11_state import RuntimeException

    print(f"Checking ONNX model loading from: {path} ...")
    try:
        onnx_options = SessionOptions()
        _ = InferenceSession(path.as_posix(), onnx_options, providers=["CPUExecutionProvider"])
        print(f"Model {path} correctly loaded: \N{heavy check mark}")
    except RuntimeException as re:
        print(f"Error while loading the model {re}: \N{heavy ballot x}")


if __name__ == "__main__":
    parser = OnnxConverterArgumentParser()
    args = parser.parse_args()

    # Make sure output is absolute path
    args.output = Path(args.output).absolute()

    try:
        print("\n====== Converting model to ONNX ======")
        # Convert
        convert(
            args.framework,
            args.model,
            args.output,
            args.opset,
            args.tokenizer,
            args.use_external_format,
            args.pipeline,
        )

        if args.quantize:
            # Ensure requirements for quantization on onnxruntime is met
            check_onnxruntime_requirements(ORT_QUANTIZE_MINIMUM_VERSION)

            # onnxruntime optimizations doesn't provide the same level of performances on TensorFlow than PyTorch
            if args.framework == "tf":
                print(
                    "\t Using TensorFlow might not provide the same optimization level compared to PyTorch.\n"
                    "\t For TensorFlow users you can try optimizing the model directly through onnxruntime_tools.\n"
                    "\t For more information, please refer to the onnxruntime documentation:\n"
                    "\t\thttps://github.com/microsoft/onnxruntime/tree/master/onnxruntime/python/tools/transformers\n"
                )

            print("\n====== Optimizing ONNX model ======")

            # Quantization works best when using the optimized version of the model
            args.optimized_output = optimize(args.output)

            # Do the quantization on the right graph
            args.quantized_output = quantize(args.optimized_output)

        # And verify
        if args.check_loading:
            print("\n====== Check exported ONNX model(s) ======")
            verify(args.output)

            if hasattr(args, "optimized_output"):
                verify(args.optimized_output)

            if hasattr(args, "quantized_output"):
                verify(args.quantized_output)

    except Exception as e:
        print(f"Error while converting the model: {e}")
        exit(1)



# coding=utf-8
# Copyright 2018 The HuggingFace Inc. team.
# Copyright (c) 2018, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
    Benchmarking the library on inference and training in PyTorch.
"""


import timeit
from typing import Callable, Optional

from ..configuration_utils import PretrainedConfig
from ..file_utils import is_py3nvml_available, is_torch_available
from ..models.auto.modeling_auto import MODEL_MAPPING, MODEL_WITH_LM_HEAD_MAPPING
from ..utils import logging
from .benchmark_utils import (
    Benchmark,
    Memory,
    MemorySummary,
    measure_peak_memory_cpu,
    start_memory_tracing,
    stop_memory_tracing,
)


if is_torch_available():
    import torch

    from .benchmark_args import PyTorchBenchmarkArguments


if is_py3nvml_available():
    import py3nvml.py3nvml as nvml


logger = logging.get_logger(__name__)


class PyTorchBenchmark(Benchmark):

    args: PyTorchBenchmarkArguments
    configs: PretrainedConfig
    framework: str = "PyTorch"

    @property
    def framework_version(self):
        return torch.__version__

    def _inference_speed(self, model_name: str, batch_size: int, sequence_length: int) -> float:
        _inference = self._prepare_inference_func(model_name, batch_size, sequence_length)
        return self._measure_speed(_inference)

    def _inference_memory(
        self, model_name: str, batch_size: int, sequence_length: int
    ) -> [Memory, Optional[MemorySummary]]:
        _inference = self._prepare_inference_func(model_name, batch_size, sequence_length)
        return self._measure_memory(_inference)

    def _train_speed(self, model_name: str, batch_size: int, sequence_length: int) -> float:
        _train = self._prepare_train_func(model_name, batch_size, sequence_length)
        return self._measure_speed(_train)

    def _train_memory(
        self, model_name: str, batch_size: int, sequence_length: int
    ) -> [Memory, Optional[MemorySummary]]:
        _train = self._prepare_train_func(model_name, batch_size, sequence_length)
        return self._measure_memory(_train)

    def _prepare_inference_func(self, model_name: str, batch_size: int, sequence_length: int) -> Callable[[], None]:
        config = self.config_dict[model_name]

        if self.args.torchscript:
            config.torchscript = True

        has_model_class_in_config = (
            hasattr(config, "architectures")
            and isinstance(config.architectures, list)
            and len(config.architectures) > 0
        )
        if not self.args.only_pretrain_model and has_model_class_in_config:
            try:
                model_class = config.architectures[0]
                transformers_module = __import__("transformers", fromlist=[model_class])
                model_cls = getattr(transformers_module, model_class)
                model = model_cls(config)
            except ImportError:
                raise ImportError(
                    f"{model_class} does not exist. If you just want to test the pretrained model, you might want to set `--only_pretrain_model` or `args.only_pretrain_model=True`."
                )
        else:
            model = MODEL_MAPPING[config.__class__](config)

        model.eval()
        model.to(self.args.device)

        # encoder-decoder has vocab size saved differently
        vocab_size = config.vocab_size if hasattr(config, "vocab_size") else config.encoder.vocab_size
        input_ids = torch.randint(vocab_size, (batch_size, sequence_length), dtype=torch.long, device=self.args.device)

        if self.args.fp16:
            logger.info("Running training in Mixed Precision...")
            if not self.args.is_gpu:
                raise ValueError("Mixed precision is possible only for GPU.")
            # amp seems to have memory leaks so that memory usage
            # is measured using .half() for now https://github.com/NVIDIA/apex/issues/439
            model.half()

        if self.args.torchscript:
            with torch.no_grad():
                inference_model = torch.jit.trace(model, input_ids)
        else:
            inference_model = model

        def encoder_decoder_forward():
            with torch.no_grad():
                outputs = inference_model(input_ids, decoder_input_ids=input_ids)
            return outputs

        def encoder_forward():
            with torch.no_grad():
                outputs = inference_model(input_ids)
            return outputs

        _forward = encoder_decoder_forward if config.is_encoder_decoder else encoder_forward
        return _forward

    def _prepare_train_func(self, model_name: str, batch_size: int, sequence_length: int) -> Callable[[], None]:
        config = self.config_dict[model_name]

        has_model_class_in_config = (
            hasattr(config, "architectures")
            and isinstance(config.architectures, list)
            and len(config.architectures) > 0
        )
        if not self.args.only_pretrain_model and has_model_class_in_config:
            try:
                model_class = config.architectures[0]
                transformers_module = __import__("transformers", fromlist=[model_class])
                model_cls = getattr(transformers_module, model_class)
                model = model_cls(config)
            except ImportError:
                raise ImportError(
                    f"{model_class} does not exist. If you just want to test the pretrained model, you might want to set `--only_pretrain_model` or `args.only_pretrain_model=True`."
                )
        else:
            model = MODEL_WITH_LM_HEAD_MAPPING[config.__class__](config)

        if self.args.torchscript:
            raise NotImplementedError("Training for torchscript is currently not implemented")
        else:
            train_model = model

        model.train()
        model.to(self.args.device)

        # encoder-decoder has vocab size saved differently
        vocab_size = config.vocab_size if hasattr(config, "vocab_size") else config.encoder.vocab_size
        input_ids = torch.randint(vocab_size, (batch_size, sequence_length), dtype=torch.long, device=self.args.device)

        if self.args.fp16:
            logger.info("Running training in Mixed Precision...")
            if not self.args.is_gpu:
                raise ValueError("Mixed precision is possible only for GPU.")

            # amp seems to have memory leaks so that memory usage
            # is measured using .half() for now https://github.com/NVIDIA/apex/issues/439
            model.half()

        def compute_loss_and_backprob_encoder():
            loss = train_model(input_ids, labels=input_ids)[0]
            loss.backward()
            return loss

        def compute_loss_and_backprob_encoder_decoder():
            loss = train_model(input_ids, decoder_input_ids=input_ids, labels=input_ids)[0]
            loss.backward()
            return loss

        _train = (
            compute_loss_and_backprob_encoder_decoder
            if config.is_encoder_decoder
            else compute_loss_and_backprob_encoder
        )
        return _train

    def _measure_speed(self, func) -> float:
        try:
            if self.args.is_tpu or self.args.torchscript:
                # run additional 10 times to stabilize compilation for tpu and torchscript
                logger.info("Do inference on TPU or torchscript. Running model 5 times to stabilize compilation")
                timeit.repeat(
                    func,
                    repeat=1,
                    number=5,
                )

            # as written in https://docs.python.org/2/library/timeit.html#timeit.Timer.repeat, min should be taken rather than the average
            runtimes = timeit.repeat(
                func,
                repeat=self.args.repeat,
                number=10,
            )

            if self.args.is_tpu and self.args.torch_xla_tpu_print_metrics:
                import torch_xla.debug.metrics as met

                self.print_fn(met.metrics_report())

            return min(runtimes) / 10.0
        except RuntimeError as e:
            self.print_fn(f"Doesn't fit on GPU. {e}")
            return "N/A"

    def _measure_memory(self, func: Callable[[], None]) -> [Memory, MemorySummary]:
        try:
            if self.args.trace_memory_line_by_line:
                trace = start_memory_tracing("transformers")

            if self.args.is_tpu:
                # tpu
                raise NotImplementedError(
                    "Memory Benchmarking is currently not implemented for TPU. Please disable memory benchmarking with `--no-memory` or `args.memory=False`"
                )
            elif self.args.is_gpu:
                if not is_py3nvml_available():
                    logger.warning(
                        "py3nvml not installed, we won't log GPU memory usage. "
                        "Install py3nvml (pip install py3nvml) to log information about GPU."
                    )
                    memory = "N/A"
                else:
                    logger.info(
                        "Measuring total GPU usage on GPU device. Make sure to not have additional processes running on the same GPU."
                    )
                    # init nvml
                    nvml.nvmlInit()
                    func()
                    handle = nvml.nvmlDeviceGetHandleByIndex(self.args.device_idx)
                    meminfo = nvml.nvmlDeviceGetMemoryInfo(handle)
                    max_bytes_in_use = meminfo.used
                    memory = Memory(max_bytes_in_use)
                    # shutdown nvml
                    nvml.nvmlShutdown()
            else:
                # cpu
                memory_bytes = measure_peak_memory_cpu(func)
                memory = Memory(memory_bytes) if isinstance(memory_bytes, int) else memory_bytes

            if self.args.trace_memory_line_by_line:
                summary = stop_memory_tracing(trace)
            else:
                summary = None

            return memory, summary
        except RuntimeError as e:
            self.print_fn(f"Doesn't fit on GPU. {e}")
            return "N/A", None
